#generates hg19_splits for cross-validation
#hg19 (i.e. /mnt/data/annotations/by_release/hg19.GRCh37/hg19.chrom.sizes). Any chromosome from this chrom.sizes file that is not in the test/validation split is assumed to be in the training split (only considering chroms 1 - 22, X, Y
hg19_chroms=['chr1','chr2','chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20','chr21','chr22','chrX','chrY']
hg19_splits=dict()
hg19_splits[0]={'test':['chr1'],
           'valid':['chr10','chr8']}
hg19_splits[1]={'test':['chr19','chr2'],
           'valid':['chr1']}
hg19_splits[2]={'test':['chr3','chr20'],
           'valid':['chr19','chr2']}
hg19_splits[3]={'test':['chr13','chr6','chr22'],
           'valid':['chr3','chr20']}
hg19_splits[4]={'test':['chr5','chr16','chrY'],
           'valid':['chr13','chr6','chr22']}
hg19_splits[5]={'test':['chr4','chr15','chr21'],
           'valid':['chr5','chr16','chrY']}
hg19_splits[6]={'test':['chr7','chr18','chr14'],
           'valid':['chr4','chr15','chr21']}
hg19_splits[7]={'test':['chr11','chr17','chrX'],
           'valid':['chr7','chr18','chr14']}
hg19_splits[8]={'test':['chr12','chr9'],
           'valid':['chr11','chr17','chrX']}
hg19_splits[9]={'test':['chr10','chr8'],
           'valid':['chr12','chr9']}


#Note: the splits for hg19 and hg38 are the same, as are the chromosomes used for training models. 
splits=dict()
splits['hg19']=hg19_splits 
splits['hg38']=hg19_splits
chroms=dict()
chroms['hg19']=hg19_chroms
chroms['hg38']=hg19_chroms



from keras.utils import Sequence
import pandas as pd
import numpy as np
import random
import math 
import pysam
from .util import ltrdict
import threading
import pickle
import pdb

def get_weights(data):
    w1=[float(data.shape[0])/sum(data.iloc[:,i]==1) for i in range(data.shape[1])]
    w0=[float(data.shape[0])/sum(data.iloc[:,i]==0) for i in range(data.shape[1])]
    return w1,w0
    

def dinuc_shuffle(seq):
    #get list of dinucleotides
    nucs=[]
    for i in range(0,len(seq),2):
        nucs.append(seq[i:i+2])
    #generate a random permutation
    random.shuffle(nucs)
    return ''.join(nucs) 


def revcomp(seq):
    seq=seq[::-1].upper()
    comp_dict=dict()
    comp_dict['A']='T'
    comp_dict['T']='A'
    comp_dict['C']='G'
    comp_dict['G']='C'
    rc=[]
    for base in seq:
        if base in comp_dict:
            rc.append(comp_dict[base])
        else:
            rc.append(base)
    return ''.join(rc)

def get_probability_thresh_for_precision(truth,predictions,precision_thresh):
    from sklearn.metrics import precision_recall_curve
    num_tasks=truth.shape[1]
    precision_thresholds=[] 
    for task_index in range(num_tasks):
        truth_task=truth.iloc[:,task_index]
        pred_task=predictions[:,task_index]
        non_ambig=truth_task!=-1        
        precision,recall,threshold=precision_recall_curve(truth_task[non_ambig],pred_task[non_ambig])
        threshold=np.insert(threshold,threshold.shape[0],1)
        merged_prc=pd.DataFrame({'precision':precision,
                                 'recall':recall,
                                 'threshold':threshold})
        precision_thresholds.append(np.min(merged_prc[merged_prc['precision']>=precision_thresh]['threshold']))
    print(precision_thresholds) 
    return precision_thresholds
    
def open_data_file(data_path,tasks,chroms_to_use):
    if data_path.endswith('.hdf5'):
        if tasks==None:
            data=pd.read_hdf(data_path)
        else:
            data=pd.read_hdf(data_path,columns=tasks)
    else:
        #treat as bed file 
        if tasks==None:
            data=pd.read_csv(data_path,header=0,sep='\t',index_col=[0,1,2])
        else:
            data=pd.read_csv(data_path,header=0,sep='\t',nrows=1)
            chrom_col=data.columns[0]
            start_col=data.columns[1]
            end_col=data.columns[2]
            data=pd.read_csv(data_path,header=0,sep='\t',usecols=[chrom_col,start_col,end_col]+tasks,index_col=[0,1,2])
    print("loaded labels") 
    if chroms_to_use!=None:
        data=data[np.in1d(data.index.get_level_values(0), chroms_to_use)]
    print("filtered on chroms_to_use")
    return data 

class TruePosGenerator(Sequence):
    def __init__(self,data_pickle,ref_fasta,batch_size=128,precision_thresh=0.9):
        f=open(data_pickle,'rb')
        data=pickle.load(f)
        self.predictions=data[0]
        self.labels=data[1]
        self.columns=self.labels.columns
        #calculate prediction probability cutoff to achieve the specified precision threshold
        self.prob_thresholds=get_probability_thresh_for_precision(self.labels,self.predictions,precision_thresh)
        truth_pred_product=self.labels*(self.predictions>=self.prob_thresholds)
        true_pos_rows=truth_pred_product[truth_pred_product.max(axis=1)>0]
        self.data=true_pos_rows
        self.indices=np.arange(self.data.shape[0])
        self.add_revcomp=False
        self.ref_fasta=ref_fasta
        self.lock=threading.Lock()
        self.batch_size=batch_size

        
    def __len__(self):
        return math.ceil(self.data.shape[0]/self.batch_size)

    def __getitem__(self,idx):
        with self.lock:
            self.ref=pysam.FastaFile(self.ref_fasta)
            return self.get_basic_batch(idx)
        
    def get_basic_batch(self,idx):
        #get seq positions
        inds=self.indices[idx*self.batch_size:(idx+1)*self.batch_size]
        bed_entries=self.data.index[inds]
        #get sequences
        seqs=[self.ref.fetch(i[0],i[1],i[2]) for i in bed_entries]
        #one-hot-encode the fasta sequences 
        seqs=np.array([[ltrdict.get(x,[0,0,0,0]) for x in seq] for seq in seqs])
        x_batch=np.expand_dims(seqs,1)
        #extract the labels at the current batch of indices 
        y_batch=np.asarray(self.data.iloc[inds])
        return (bed_entries,x_batch,y_batch)    


#use wrappers for keras Sequence generator class to allow batch shuffling upon epoch end
class DataGenerator(Sequence):
    def __init__(self,data_path,ref_fasta,batch_size=128,add_revcomp=True,tasks=None,shuffled_ref_negatives=False,upsample=True,upsample_ratio=0.1,chroms_to_use=None,get_w1_w0=False):
        self.lock = threading.Lock()        
        self.batch_size=batch_size
        #decide if reverse complement should be used
        self.add_revcomp=add_revcomp
        if add_revcomp==True:
            self.batch_size=int(batch_size/2)

        #determine whether negative set should consist of the shuffled refs.
        # If so, split batch size in 2, as each batch will be augmented with shuffled ref negatives
        # in ratio equal to positives 
        self.shuffled_ref_negatives=shuffled_ref_negatives
        if self.shuffled_ref_negatives==True:
            self.batch_size=int(self.batch_size/2)
    
        #open the reference file
        self.ref_fasta=ref_fasta

        self.data=open_data_file(data_path,tasks,chroms_to_use)
        if get_w1_w0==True:
            w1,w0=get_weights(self.data)
            self.w1=w1
            self.w0=w0
            print(self.w1)
        self.indices=np.arange(self.data.shape[0])
        num_indices=self.indices.shape[0]
        self.add_revcomp=add_revcomp
        
        #set variables needed for upsampling the positives
        self.upsample=upsample
        if self.upsample==True:
            self.upsample_ratio=upsample_ratio
            self.ones = self.data.loc[(self.data > 0).any(axis=1)]
            self.zeros = self.data.loc[(self.data < 1).all(axis=1)]
            self.pos_batch_size = int(self.batch_size * self.upsample_ratio)
            self.neg_batch_size = self.batch_size - self.pos_batch_size
            self.pos_indices=np.arange(self.ones.shape[0])
            self.neg_indices=np.arange(self.zeros.shape[0])
            
            #wrap the positive and negative indices to reach size of self.indices
            num_pos_wraps=math.ceil(num_indices/self.pos_indices.shape[0])
            num_neg_wraps=math.ceil(num_indices/self.neg_indices.shape[0])
            self.pos_indices=np.repeat(self.pos_indices,num_pos_wraps)[0:num_indices]
            np.random.shuffle(self.pos_indices)
            self.neg_indices=np.repeat(self.neg_indices,num_neg_wraps)[0:num_indices]
            np.random.shuffle(self.neg_indices)
            
    def __len__(self):
        return math.ceil(self.data.shape[0]/self.batch_size)

    def __getitem__(self,idx):
        with self.lock:
            ref=pysam.FastaFile(self.ref_fasta)
            self.ref=ref
            if self.shuffled_ref_negatives==True:
                return self.get_shuffled_ref_negatives_batch(idx)
            elif self.upsample==True:
                return self.get_upsampled_positives_batch(idx)
            else:
                return self.get_basic_batch(idx) 
        
    def get_shuffled_ref_negatives_batch(self,idx): 
        #get seq positions
        inds=self.indices[idx*self.batch_size:(idx+1)*self.batch_size]
        bed_entries=self.data.index[inds]
        #get sequences
        seqs=[self.ref.fetch(i[0],i[1],i[2]) for i in bed_entries]
        if self.add_revcomp==True:
            #add in the reverse-complemented sequences for training.
            seqs_rc=[revcomp(s) for s in seqs]
            seqs=seqs+seqs_rc
            
        #generate the corresponding negative set by dinucleotide-shuffling the sequences
        seqs_shuffled=[dinuc_shuffle(s) for s in seqs]
        seqs=seqs+seqs_shuffled
        #one-hot-encode the fasta sequences 
        seqs=np.array([[ltrdict.get(x,[0,0,0,0]) for x in seq] for seq in seqs])
        x_batch=np.expand_dims(seqs,1)
        y_batch=np.asarray(self.data.iloc[inds])
        if self.add_revcomp==True:
            y_batch=np.concatenate((y_batch,y_batch),axis=0)
        y_shape=y_batch.shape 
        y_batch=np.concatenate((y_batch,np.zeros(y_shape)))
        return (x_batch,y_batch)

    def get_upsampled_positives_batch(self,idx):
        #get seq positions
        pos_inds=self.pos_indices[idx*self.pos_batch_size:(idx+1)*self.pos_batch_size]
        pos_bed_entries=self.ones.index[pos_inds]
        neg_inds=self.neg_indices[idx*self.neg_batch_size:(idx+1)*self.neg_batch_size]
        neg_bed_entries=self.zeros.index[neg_inds]
    
        #print(neg_inds[0:10])
        #bed_entries=pos_bed_entries+neg_bed_entries

        #get sequences
        pos_seqs=[self.ref.fetch(i[0],i[1],i[2]) for i in pos_bed_entries]
        neg_seqs=[self.ref.fetch(i[0],i[1],i[2]) for i in neg_bed_entries]
        seqs=pos_seqs+neg_seqs 
        if self.add_revcomp==True:
            #add in the reverse-complemented sequences for training.
            seqs_rc=[revcomp(s) for s in seqs]
            seqs=seqs+seqs_rc
            
        #one-hot-encode the fasta sequences 
        seqs=np.array([[ltrdict.get(x,[0,0,0,0]) for x in seq] for seq in seqs])
        x_batch=np.expand_dims(seqs,1)
        
        #extract the positive and negative labels at the current batch of indices
        y_batch_pos=self.ones.iloc[pos_inds]
        y_batch_neg=self.zeros.iloc[neg_inds]
        y_batch=np.concatenate((y_batch_pos,y_batch_neg),axis=0)
        #add in the labels for the reverse complement sequences, if used 
        if self.add_revcomp==True:
            y_batch=np.concatenate((y_batch,y_batch),axis=0)
        return (x_batch,y_batch)            
    
    def get_basic_batch(self,idx):
        #get seq positions
        inds=self.indices[idx*self.batch_size:(idx+1)*self.batch_size]
        bed_entries=self.data.index[inds]
        #get sequences
        seqs=[self.ref.fetch(i[0],i[1],i[2]) for i in bed_entries]
        if self.add_revcomp==True:
            #add in the reverse-complemented sequences for training.
            seqs_rc=[revcomp(s) for s in seqs]
            seqs=seqs+seqs_rc
        #one-hot-encode the fasta sequences 
        seqs=np.array([[ltrdict.get(x,[0,0,0,0]) for x in seq] for seq in seqs])
        x_batch=np.expand_dims(seqs,1)
        #extract the labels at the current batch of indices 
        y_batch=np.asarray(self.data.iloc[inds])
        #add in the labels for the reverse complement sequences, if used 
        if self.add_revcomp==True:
            y_batch=np.concatenate((y_batch,y_batch),axis=0)
        return (x_batch,y_batch)
    
    def on_epoch_end(self):
        #if upsampling is being used, shuffle the positive and negative indices 
        if self.upsample==True:
            np.random.shuffle(self.pos_indices)
            np.random.shuffle(self.neg_indices)
        else:
            np.random.shuffle(self.indices)



import pysam
import pybedtools
import pandas as pd
import numpy as np 
import pdb

ref="/srv/scratch/annashch/deeplearning/form_inputs/code/hg19.genome.fa"
fasta_ref = pysam.FastaFile(ref)
train_path="/srv/scratch/annashch/deeplearning/form_inputs/gecco_inputs_v2/gecco_v2.train.bed"
#load the train data as a pandas dataframe, skip the header
data=pd.read_csv(train_path,header=0,sep='\t',index_col=[0,1,2])
ltrdict = {'a':[1,0,0,0],'c':[0,1,0,0],'g':[0,0,1,0],'t':[0,0,0,1], 'n':[0,0,0,0],'A':[1,0,0,0],'C':[0,1,0,0],'G':[0,0,1,0],'T':[0,0,0,1],'N':[0,0,0,0]}
#iterate through batches and one-hot-encode on the fly
start_index=75000
num_generated=77009
total_entries=data.shape[0]
batch_size=1000
while True:
        if(num_generated >=total_entries):
                start_index=0
        end_index=start_index+batch_size
        #get seq positions
        bed_entries=[(data.index[i]) for i in range(start_index,end_index)]
        #get sequences
        seqs=[fasta_ref.fetch(i[0],i[1],i[2]) for i in bed_entries]
        seqs=np.array([[ltrdict[x] for x in seq] for seq in seqs])
        #expand dimension of 1
        x_batch=np.expand_dims(seqs,1)
        y_batch=data[start_index:end_index]
        num_generated+=batch_size
        start_index=end_index
        print('done!')









import keras.backend as K
import tensorflow as tf
import pdb 
import numpy as np 
def get_weighted_binary_crossentropy(w0_weights, w1_weights,ambig_val=np.nan):
    # Compute the task-weighted cross-entropy loss, where every task is weighted by 1 - (fraction of non-ambiguous examples that are positive)
    # In addition, weight everything with label -1 to 0
    import numpy as np 
    w0_weights=np.array(w0_weights);
    w1_weights=np.array(w1_weights);
    def weighted_binary_crossentropy(y_true,y_pred):
        weightsPerTaskRep = y_true*w1_weights[None,:] + (1-y_true)*w0_weights[None,:]
        nonAmbig=tf.math.logical_not(tf.is_nan(y_true))
        nonAmbigTimesWeightsPerTask = tf.boolean_mask(weightsPerTaskRep,nonAmbig)
        return K.mean(K.binary_crossentropy(tf.boolean_mask(y_true,nonAmbig),tf.boolean_mask(y_pred,nonAmbig))*nonAmbigTimesWeightsPerTask, axis=-1);
    return weighted_binary_crossentropy; 

def get_ambig_binary_crossentropy():
    def ambig_binary_crossentropy(y_true,y_pred):
        nonAmbig=tf.math.logical_not(tf.is_nan(y_true))
        return K.mean(K.binary_crossentropy(tf.boolean_mask(y_true,nonAmbig), tf.boolean_mask(y_pred,nonAmbig)), axis=-1);
    return ambig_binary_crossentropy; 

def get_ambig_mean_squared_error(ambig_val=np.nan): 
    def ambig_mean_squared_error(y_true, y_pred):
        nonAmbig=tf.math.logical_not(tf.is_nan(y_true))
        return K.mean(K.square(tf.boolean_mask(y_pred,nonAmbig) - tf.boolean_mask(y_true,nonAmbig)), axis=-1)
    return ambig_mean_squared_error



#compute gradient x input for tensorflow models.
import argparse
import pdb
import pickle
from deeplift.conversion import kerasapi_conversion as kc
import keras.backend as K
import numpy as np
import pyBigWig
from .config import args_object_from_args_dict
from .generators import *
from .predict import get_model

def parse_args():
    parser=argparse.ArgumentParser(description="get gradient x input for a model",add_help=True)
    parser.add_argument("--model_hdf5")
    parser.add_argument("--w0",nargs="+",type=float,default=None)
    parser.add_argument("--w1",nargs="+",type=float,default=None)
    parser.add_argument("--w1_w0_file",default=None)
    parser.add_argument("--predictions_pickle")
    parser.add_argument("--interpret_chroms",nargs="*") 
    parser.add_argument("--interpretation_outf",default=None)
    parser.add_argument("--flank",default=None,type=int)
    parser.add_argument("--method",choices=['gradxinput','deeplift'],default="deeplift")
    parser.add_argument('--batch_size',type=int,help='batch size to use to make model predictions',default=50)
    parser.add_argument('--ref_fasta',default="/mnt/data/annotations/by_release/hg19.GRCh37/hg19.genome.fa")
    parser.add_argument('--background_freqs',default=None)
    parser.add_argument('--center_on_summit',default=False,action='store_true',help="if this is set to true, the peak will be centered at the summit (must be last entry in bed file or hammock) and expanded args.flank to the left and right")
    parser.add_argument('--task_id',type=int)
    parser.add_argument('--squeeze_input_for_gru',default=False,action='store_true')
    parser.add_argument('--assembly',default='hg19')
    parser.add_argument('--chromsizes',default='/mnt/data/annotations/by_release/hg19.GRCh37/hg19.chrom.sizes')
    parser.add_argument("--precision_thresh",type=float,default=0.9,help="threshold for precision that is used in determining the probability cutoff to use in calling positive predictions")
    parser.add_argument("--yaml",default=None)
    parser.add_argument("--json",default=None)
    
    return parser.parse_args()


def get_deeplift_function(args):
    # convert to deeplift model and get scoring function
    deeplift_model = kc.convert_model_from_saved_files(args.model_hdf5,verbose=False)

    #get the deeplift score with respect to the logit 
    score_func = deeplift_model.get_target_contribs_func(
        find_scores_layer_idx=0,
        target_layer_idx=-2)
    return score_func

def get_deeplift_references(args):
    if args.background_freqs==None:
        # use a 40% GC reference
        input_references = [np.array([0.3, 0.2, 0.2, 0.3])[None, None, None, :]]
    else:
        input_references=[np.array(args.background_freqs)[None,None,None,:]]
    return input_references

def add_bigwig_header(bw,assembly):
    if assembly=='hg19':
        bw.addHeader([('chr1',249250621),
                      ('chr2',243199373),                      
                      ('chr3',198022430),
                      ('chr4',191154276),
                      ('chr5',180915260),
                      ('chr6',171115067),
                      ('chr7',159138663),
                      ('chr8',146364022),
                      ('chr9',141213431),
                      ('chr10',135534747),
                      ('chr11',135006516),
                      ('chr12',133851895),
                      ('chr13',115169878),
                      ('chr14',107349540),
                      ('chr15',102531392),
                      ('chr16',90354753),
                      ('chr17',81195210),
                      ('chr18',78077248),
                      ('chr19',59128983),
                      ('chr20',63025520),
                      ('chr21',48129895),
                      ('chr22',51304566),
                      ('chrX',155270560),
                      ('chrY',59373566)])
        return bw
    else:
        raise Exception("implement bigWig header for this assembly!")

def get_chromsizes(f):
    data=open(f,'r').read().strip().split('\n')
    chromsize_dict=dict()
    for line in data:
        tokens=line.split()
        chromsize_dict[tokens[0]]=int(tokens[1])
    return chromsize_dict



def interpret(args):
    print("starting interpretation")
    if type(args)==type({}):
        args=args_object_from_args_dict(args)
    if args.method=="deeplift":
        # get deeplift scores
        score_func=get_deeplift_function(args)
        input_references=get_deeplift_references(args)
    elif args.method=="gradxinput":
        #calculate gradient x input 
        model=get_model(args)
        grad_tensor=K.gradients(model.layers[-2].output,model.layers[0].input)
        grad_func = K.function([model.layers[0].input,K.learning_phase()], grad_tensor)
    else:
        raise Exception("method must be one of 'deeplift' or 'gradxinput'")
    
    data_generator=TruePosGenerator(args.predictions_pickle,
                                        args.ref_fasta,
                                        batch_size=args.batch_size,
                                        precision_thresh=args.precision_thresh)
    print("made data generator!") 
    tasks=list(data_generator.columns)
    bigwigs={}
    for task in tasks:
        bw=pyBigWig.open(args.interpretation_outf+"."+task,'w')
        add_bigwig_header(bw,args.assembly)
        bigwigs[task]=bw
    print("initialized output bigwig files") 
    chromsize_dict=get_chromsizes(args.chromsizes)
    
    #iterate through the batches and get interpretation scores for each task
    num_tasks=len(tasks)
    num_batches=len(data_generator)
    for i in range(num_batches):
        print(str(i)+"/"+str(num_batches)) 
        bed_entries,x,y=data_generator[i]
        for t_index in range(num_tasks):
            cur_task=tasks[t_index]
            if args.method=="deeplift":
                # get deeplift scores
                cur_scores = score_func(
                    task_idx=t_index,
                    input_data_list=[x],
                    batch_size=x.shape[0],
                    progress_update=None,
                    input_references_list=input_references)*x
            else:
                gradient = grad_func([x, False])[0]
                normed_gradient = gradient-np.mean(gradient, axis=3)[:,:,:,None]
                cur_scores = normed_gradient*x
            #save to bigwig 
            chroms = [entry[0] for entry in bed_entries]
            start_vals=[entry[1] for entry in bed_entries] 
            for score_index in range(len(cur_scores)):
                base_scores=np.ndarray.tolist(np.sum(cur_scores[score_index].squeeze(),axis=1))
                try:
                    bigwigs[cur_task].addEntries(chroms[score_index],start_vals[score_index],values=base_scores,span=1,step=1)
                except:
                    continue
    for cur_task in tasks:
        bigwigs[cur_task].close()
            
def main():
    args=parse_args()
    interpret(args) 
    
if __name__=="__main__":
    main()
    



from sklearn.metrics import roc_auc_score
from sklearn.metrics import average_precision_score, precision_recall_curve 
from scipy.stats import spearmanr, pearsonr 
import numpy as np
from .util import enum 
from collections import OrderedDict, defaultdict
import sys 
import pdb
import tensorflow as tf
import keras.backend as K 

def remove_ambiguous_peaks(predictions, true_y,ambig_val=np.nan): 
    indices_to_remove = np.nonzero(np.isnan(true_y))
    true_y_filtered = np.delete(true_y, indices_to_remove)
    predictions_filtered = np.delete(predictions, indices_to_remove)
    return predictions_filtered, true_y_filtered



def auroc_func(predictions_for_task_filtered, true_y_for_task_filtered):
    try:
        task_auroc = roc_auc_score(y_true=true_y_for_task_filtered,
                                   y_score=predictions_for_task_filtered)
    except Exception as e:
        #if there is only one class in the batch of true_y, then auROC cannot be calculated
        print("Could not calculate auROC:")
        print(str(e))
        task_auroc=None 
    return task_auroc

def auprc_func(predictions_for_task_filtered, true_y_for_task_filtered):
    # sklearn only supports 2 classes (0,1) for the auPRC calculation
    try:
        task_auprc=average_precision_score(true_y_for_task_filtered, predictions_for_task_filtered)
    except:
        print("Could not calculate auPRC:")
        print(sys.exc_info()[0])
        task_auprc=None 
    return task_auprc

    

def get_accuracy_stats_for_task(predictions_for_task_filtered, true_y_for_task_filtered, c):
    predictions_for_task_filtered_round = np.array([round(el) for el in predictions_for_task_filtered])
    accuratePredictions = predictions_for_task_filtered_round==true_y_for_task_filtered;

    numPositives_forTask=np.sum(true_y_for_task_filtered==1,axis=0,dtype="float");
    numNegatives_forTask=np.sum(true_y_for_task_filtered==0,axis=0,dtype="float"); 

    accuratePredictions_positives = np.sum(accuratePredictions*(true_y_for_task_filtered==1),axis=0);
    accuratePredictions_negatives = np.sum(accuratePredictions*(true_y_for_task_filtered==0),axis=0);
    unbalancedAccuracy_forTask = (accuratePredictions_positives + accuratePredictions_negatives)/(numPositives_forTask + numNegatives_forTask)

    positiveAccuracy_forTask = accuratePredictions_positives/numPositives_forTask
    negativeAccuracy_forTask = accuratePredictions_negatives/numNegatives_forTask
    balancedAccuracy_forTask= (positiveAccuracy_forTask+negativeAccuracy_forTask)/2;
    returnDict={'unbalanced_accuracy':unbalancedAccuracy_forTask,
                'positive_accuracy':positiveAccuracy_forTask,
                'negative_accuracy':negativeAccuracy_forTask,
                'balanced_accuracy':balancedAccuracy_forTask,
                'num_positives':numPositives_forTask,
                'num_negatives':numNegatives_forTask}
    return returnDict


def recall_at_fdr_function(predictions_for_task_filtered,true_y_for_task_filtered,fdr_thresh_list):
    for fdr_thresh_index in range(len(fdr_thresh_list)):
        if float(fdr_thresh_list[fdr_thresh_index])>1:
            fdr_thresh_list[fdr_thresh_index]=fdr_thresh_list[fdr_thresh_index]/100
            
    precision,recall,class_thresholds=precision_recall_curve(true_y_for_task_filtered,predictions_for_task_filtered)
    fdr=1-precision

    #remove the last values in recall and fdr, as the scipy precision_recall_curve function sets them to 0 automatically
    recall=np.delete(recall,-1)
    fdr=np.delete(fdr,-1)

    #concatenate recall,fdr, thresholds along axis=1
    recall=np.expand_dims(recall,axis=1)
    fdr=np.expand_dims(fdr,axis=1)
    class_thresholds=np.expand_dims(class_thresholds,axis=1)
    data=np.concatenate((recall,fdr,class_thresholds),axis=1)

    #sort by threshold
    data=data[data[:,2].argsort()]

    #get the recall, fdr at each thresh
    recall_thresholds=[]
    class_thresholds=[]
    for fdr_thresh in fdr_thresh_list:
        try:
            data_index=np.max(np.nonzero(data[:,1]<=fdr_thresh))
            recall_thresholds.append(data[data_index,0])
            class_thresholds.append(data[data_index,2])
        except:
            print("No class threshold can give requested fdr <=:"+str(fdr_thresh))
            recall_thresholds.append(np.nan)
            class_thresholds.append(np.nan)
            
    return recall_thresholds, class_thresholds


def get_performance_metrics_classification(predictions,true_y):
    assert predictions.shape==true_y.shape;
    assert len(predictions.shape)==2;
    [num_rows, num_cols]=true_y.shape 
    performance_stats=None
    for c in range(num_cols):
        true_y_for_task=np.squeeze(true_y[:,c])
        predictions_for_task=np.squeeze(predictions[:,c])
        predictions_for_task_filtered,true_y_for_task_filtered = remove_ambiguous_peaks(predictions_for_task,true_y_for_task)

        accuracy_stats_task = get_accuracy_stats_for_task(predictions_for_task_filtered, true_y_for_task_filtered, c)
        auprc_task=auprc_func(predictions_for_task_filtered,true_y_for_task_filtered)
        auroc_task=auroc_func(predictions_for_task_filtered,true_y_for_task_filtered)
        recall,class_thresh=recall_at_fdr_function(predictions_for_task_filtered,true_y_for_task_filtered,[50,20,10])
                
        if performance_stats==None:
            performance_stats=dict()
            for key in accuracy_stats_task:
                performance_stats[key]=[accuracy_stats_task[key]]
            performance_stats['auprc']=[auprc_task]
            performance_stats['auroc']=[auroc_task]
            performance_stats['recall_at_fdr_50']=[recall[0]]
            performance_stats['recall_at_fdr_20']=[recall[1]]
            performance_stats['recall_at_fdr_10']=[recall[2]]            
        else:
            for key in accuracy_stats_task:
                performance_stats[key].append(accuracy_stats_task[key])
            performance_stats['auprc'].append(auprc_task)
            performance_stats['auroc'].append(auroc_task)
            performance_stats['recall_at_fdr_50'].append(recall[0])
            performance_stats['recall_at_fdr_20'].append(recall[1])
            performance_stats['recall_at_fdr_10'].append(recall[2])
    print(str(performance_stats))
    return performance_stats  


def get_performance_metrics_regression(predictions,true_y):
    assert predictions.shape==true_y.shape;
    assert len(predictions.shape)==2;
    [num_rows, num_cols]=true_y.shape 
    performance_stats=None
    for c in range(num_cols):
        true_y_for_task=np.squeeze(true_y[:,c])
        predictions_for_task=np.squeeze(predictions[:,c])
        predictions_for_task_filtered,true_y_for_task_filtered = remove_ambiguous_peaks(predictions_for_task,true_y_for_task)
        spearman_task=spearmanr(predictions_for_task_filtered,true_y_for_task_filtered)
        pearson_task=pearsonr(predictions_for_task_filtered,true_y_for_task_filtered)
        #get correlation for non-zero true values
        non_zero_true=true_y_for_task_filtered[true_y_for_task_filtered!=0]
        non_zero_predicted=predictions_for_task_filtered[true_y_for_task_filtered!=0]
        spearman_task_nonzero=spearmanr(non_zero_predicted,non_zero_true)
        pearson_task_nonzero=pearsonr(non_zero_predicted,non_zero_true)
        
        if performance_stats==None:
            performance_stats={'spearmanr':[spearman_task],
                               'pearsonr':[pearson_task],
                               'spearmanr_nonzerobins':[spearman_task_nonzero],
                               'pearsonr_nonzerobins':[pearson_task_nonzero]}
        else:
            performance_stats['spearmanr'].append(spearman_task)
            performance_stats['pearsonr'].append(pearson_task)
            performance_stats['spearman_nonzerobins'].append(spearman_task_nonzero)
            performance_stats['pearson_nonzerobins'].append(pearson_task_nonzero)
            
    return performance_stats  



from kerasAC.activations import softMaxAxis1
from kerasAC.generators import *
from kerasAC.config import args_object_from_args_dict
from kerasAC.performance_metrics import *
from kerasAC.custom_losses import *
from kerasAC.metrics import recall, specificity, fpr, fnr, precision, f1
import argparse
import yaml 
import h5py 
import pickle
import numpy as np 
import keras 
from keras.losses import *
from kerasAC.custom_losses import *
from abstention.calibration import PlattScaling, IsotonicRegression 
import random
from scipy.special import logit,expit
import pdb 

def get_weights(args):
    w1=None
    w0=None
    if args.w1_w0_file!=None:
        w1_w0=np.loadtxt(args.w1_w0_file)
        w1=w1_w0[:,0]
        w0=w1_w0[:,1]
    if args.w1!=None:
        w1=args.w1
    if args.w0!=None:
        w0=args.w0 
    return w1,w0

def get_predictions_hammock(args,model):
    import pysam
    import pandas as pd
    num_generated=0
    ref=pysam.FastaFile(args.ref_fasta) 
    data=pd.read_csv(args.data_hammock,header=None,sep='\t')
    ltrdict = {'a':[1,0,0,0],'c':[0,1,0,0],'g':[0,0,1,0],'t':[0,0,0,1], 'n':[0,0,0,0],'A':[1,0,0,0],'C':[0,1,0,0],'G':[0,0,1,0],'T':[0,0,0,1],'N':[0,0,0,0]}
    #iterate through batches and one-hot-encode on the fly
    num_entries=data.shape[0]
    predictions=None
    all_names=[] 
    while num_generated < num_entries:
        print(str(num_generated))
        start_index=num_generated
        end_index=min([num_entries,start_index+args.batch_size])
        seqs=[]
        names=[]
        for i in range(start_index,end_index):
            cur_row=data.iloc[i]
            chrom=cur_row[0]
            start_val=cur_row[1]
            end_val=cur_row[2]
            peak_metadata=cur_row[3].split(',') 
            peak_name=','.join([peak_metadata[-3],peak_metadata[-2]])
            if args.center_on_summit==True:
                summit_offset=int(peak_metadata[-1].split('[')[1].split(']')[0])
                summit_pos=start_val+summit_offset
                start_val=summit_pos - args.flank
                end_val=summit_pos+args.flank
            if start_val<1:
                start_val=1
                end_val=1+2*args.flank 
            peak_name='\t'.join([str(i) for i in [chrom,start_val,end_val,peak_name]])
            names.append(peak_name)
            try:
                seq=ref.fetch(chrom,start_val,end_val)
            except:
                continue
            seqs.append(seq)
        seqs=np.array([[ltrdict.get(x,[0,0,0,0]) for x in seq] for seq in seqs])
        if (args.squeeze_input_for_gru==False):
            #expand dimension of 1
            x=np.expand_dims(seqs,1)
        else:
            x=seqs
        try:
            predictions_batch=model.predict(x)
        except:
            print("could not get predictions -- chances are reference assembly is wrong, or bed region lies outside of chrom sizes") 
        #add the batch predictions to the full set of predictions
        if type(predictions)==type(None):
            predictions=predictions_batch
        elif type(predictions)==np.ndarray:
            predictions=np.concatenate((predictions,predictions_batch),axis=0)
            print(predictions.shape)
        elif type(predictions)==type({}):
            for key in predictions_batch:
                predictions[key]=np.concatenate((predictions[key],predictions_batch[key]),axis=0)
        else:
            print("Unsupported data type for predictions: must be np.ndarray, None, or dictionary")
            pdb.set_trace()
        all_names=all_names+names 
        num_generated+=(end_index-start_index)
        
    return [predictions,data,all_names]

def get_predictions_basic(args,model):
    import pysam
    import pandas as pd
    test_generator=DataGenerator(args.data_path,args.ref_fasta,upsample=False,add_revcomp=False,batch_size=1000,chroms_to_use=args.predict_chroms)
    predictions=model.predict_generator(test_generator,
                                        max_queue_size=args.max_queue_size,
                                        workers=args.threads,
                                        use_multiprocessing=True,
                                        verbose=1)
    return [predictions,test_generator.data,None]

def get_predictions_variant(args,model):
    import pysam
    ref=pysam.FastaFile(args.ref_fasta)
    ltrdict = {'a':[1,0,0,0],'c':[0,1,0,0],'g':[0,0,1,0],'t':[0,0,0,1], 'n':[0,0,0,0],'A':[1,0,0,0],'C':[0,1,0,0],'G':[0,0,1,0],'T':[0,0,0,1],'N':[0,0,0,0]}
    #assumes the bed file has `chrom start ref alt` entries
    data=[i.split('\t') for i in open(args.variant_bed,'r').read().strip().split('\n')]
    #original
    seqs=[]
    #snp
    seqs_snp=[]
    #n bases around snp (i.e. knock out an enhancer)
    seqs_enhancer=[] 
    for entry in data[1::]:
        #introduce the specified variant 
        start_pos=int(entry[1])-(args.flank)
        end_pos=int(entry[1])+args.flank
        seq=ref.fetch(entry[0],start_pos,end_pos)
        seqs.append(seq)        
        alt_allele=entry[3]
        if alt_allele=="NA":
            #randomly insert another base
            ref_allele=seq[args.flank-1].upper()
            options=['A','C','G','T']
            options.remove(ref_allele)
            alt_allele=options[random.randint(0,2)]
        seq=seq[0:args.flank-1]+alt_allele+seq[args.flank:len(seq)]
        seqs_snp.append(seq)

        seq=np.array([ltrdict[x] for x in seq])
        start_mask=args.flank-1-args.mask
        end_mask=args.flank-1+args.mask
        seq=seq*1.0
        seq[start_mask:end_mask]=args.background_freqs
        seqs_enhancer.append(seq)
        
    seqs=np.array([[ltrdict[x] for x in seq] for seq in seqs])    
    seqs_snp=np.array([[ltrdict[x] for x in seq] for seq in seqs_snp])
    seqs_enhancer=np.asarray(seqs_enhancer)

    seqs=np.expand_dims(seqs,1)
    seqs_snp=np.expand_dims(seqs_snp,1)
    seqs_enhancer=np.expand_dims(seqs_enhancer,1)
    
    predictions_seqs=model.predict(seqs)
    predictions_seqs_snp=model.predict(seqs_snp)
    predictions_seqs_enhancer=model.predict(seqs_enhancer)
    return [predictions_seqs,predictions_seqs_snp,predictions_seqs_enhancer]

def parse_args():
    parser=argparse.ArgumentParser(description='Provide a model yaml & weights files & a dataset, get model predictions and accuracy metrics')
    parser.add_argument("--threads",type=int,default=1)
    parser.add_argument("--max_queue_size",type=int,default=100)
    parser.add_argument('--model_hdf5',help='hdf5 file that stores the model')
    parser.add_argument('--weights',help='weights file for the model')
    parser.add_argument('--yaml',help='yaml file for the model')
    parser.add_argument('--json',help='json file for the model')
    parser.add_argument('--data_path',required=True)
    parser.add_argument('--predict_chroms',nargs="*",default=None) 
    parser.add_argument('--data_hammock',help='input file is in hammock format, with unique id for each peak')
    parser.add_argument('--variant_bed')
    parser.add_argument('--predictions_pickle',help='name of pickle to save predictions',default=None)
    parser.add_argument('--performance_metrics_classification_file',help='file name to save accuracy metrics; accuracy metrics not computed if file not provided',default=None)
    parser.add_argument('--performance_metrics_regression_file',help='file name to save accuracy metrics; accuracy metrics not computed if file not provided',default=None)
    parser.add_argument('--predictions_pickle_to_load',help="if predictions have already been generated, provide a pickle with them to just compute the accuracy metrics",default=None)
    parser.add_argument('--batch_size',type=int,help='batch size to use to make model predictions',default=50)
    parser.add_argument('--functional',default=False,help='use this flag if your model is a functional model',action="store_true")
    parser.add_argument('--ref_fasta',default="/mnt/data/annotations/by_release/hg19.GRCh37/hg19.genome.fa")
    parser.add_argument('--background_freqs',default=None)
    parser.add_argument('--w1',nargs="*",type=float)
    parser.add_argument('--w0',nargs="*",type=float)
    parser.add_argument("--w1_w0_file",default=None)
    parser.add_argument('--flank',default=500,type=int)
    parser.add_argument('--mask',default=10,type=int)
    parser.add_argument('--squeeze_input_for_gru',action='store_true')
    parser.add_argument('--center_on_summit',default=False,action='store_true',help="if this is set to true, the peak will be centered at the summit (must be last entry in bed file or hammock) and expanded args.flank to the left and right")
    parser.add_argument("--calibrate_classification",action="store_true",default=False)
    parser.add_argument("--calibrate_regression",action="store_true",default=False) 
    return parser.parse_args()

def get_model(args):

    from kerasAC.metrics import recall, specificity, fpr, fnr, precision, f1
    
    custom_objects={"recall":recall,
                    "sensitivity":recall,
                    "specificity":specificity,
                    "fpr":fpr,
                    "fnr":fnr,
                    "precision":precision,
                    "f1":f1,
                    "ambig_binary_crossentropy":get_ambig_binary_crossentropy(),
                    "ambig_mean_squared_error":get_ambig_mean_squared_error()}
    w1,w0=get_weights(args)
    if type(w1) in [np.ndarray, list]: 
        loss_function=get_weighted_binary_crossentropy(w0,w1)
        custom_objects["weighted_binary_crossentropy"]=loss_function
#    try:
    if args.yaml!=None:
        from keras.models import model_from_yaml
        #load the model architecture from yaml
        yaml_string=open(args.yaml,'r').read()
        model=model_from_yaml(yaml_string,custom_objects=custom_objects) 
        #load the model weights
        model.load_weights(args.weights)
    elif args.json!=None:
        from keras.models import model_from_json
        #load the model architecture from json
        json_string=open(args.json,'r').read()
        model=model_from_json(json_string,custom_objects=custom_objects)
        model.load_weights(args.weights)
    elif args.model_hdf5!=None: 
        #load from the hdf5
        from keras.models import load_model
        model=load_model(args.model_hdf5,custom_objects=custom_objects)
    print("got model architecture")
    print("loaded model weights")        
        
#    except: 
#        print("Failed to load model. HINT: if you're using weighted binary cross entropy loss, chances are you forgot to provide the --w0 or --w1 flags")
    return model

def get_predictions(args,model):
    if args.variant_bed!=None:
        predictions=get_predictions_variant(args,model)
    elif args.data_hammock!=None:
        predictions=get_predictions_hammock(args,model) 
    else:
        predictions=get_predictions_basic(args,model) 
    print('got model predictions')
    return predictions

def calibrate(predictions,args):
    if args.calibrate_classification == True:
        #calibrate classification predictions
        #avoid -inf values and inf values:
        pseudocount=1e-5
        logit_input=predictions[0]
        logit_input[logit_input<=0]+=pseudocount
        logit_input[logit_input>=1]-=pseudocount 
        logits=logit(logit_input)
        labels=predictions[1].values
        #perform calibration for each task!
        calibrated_predictions=None
        for i in range(logits.shape[1]):
            #don't calibrate on nan inputs
            nonambiguous_indices=np.argwhere(~np.isnan(labels[:,i]))
            classification_calibration_func = PlattScaling()(
                valid_preacts=logits[nonambiguous_indices,i],
                valid_labels=labels[nonambiguous_indices,i])
            calibrated_predictions_task=classification_calibration_func(logits[:,i])
            if calibrated_predictions is None:
                calibrated_predictions=np.expand_dims(calibrated_predictions_task,axis=1)
            else:
                calibrated_predictions=np.concatenate((calibrated_predictions,np.expand_dims(calibrated_predictions_task,axis=1)),axis=1)
        predictions.append(logits)
        predictions.append(calibrated_predictions)        
        print("predictions calibrated with Platt scaling")

    elif args.calibrate_regression==True:
        regression_calibration_func=IsotonicRegression()(
            valid_preacts=predictions[0],
            valid_labels=predictions[1].values)
        calibrated_predictions=regression_calibration_func(predictions[0])
        predictions.append(None)
        predictions.append(calibrated_predictions)
        print("predictions calibrated with Isotonic Regression")
    return predictions

def predict(args):
    if type(args)==type({}):
        args=args_object_from_args_dict(args) 
    
    #get the predictions
    if args.predictions_pickle_to_load!=None:
        #load the pickled predictions
        with open(args.predictions_pickle_to_load,'rb') as handle:
            predictions=pickle.load(handle)
        print("loaded predictions from pickle") 
    else:
        #get the model
        model=get_model(args)
        predictions=get_predictions(args,model)
        assert not ((args.calibrate_classification==True) and (args.calibrate_regression==True))
        if args.predictions_pickle!=None:
            #pickle the predictions in case an error occurs downstream
            #this will allow for easy recovery of model predictions without having to regenerate them
            with open(args.predictions_pickle,'wb') as handle:
                pickle.dump(predictions,handle,protocol=pickle.HIGHEST_PROTOCOL)
            print("pickled the model predictions to file:"+str(args.predictions_pickle))
        predictions=calibrate(predictions,args)
    
    if ((args.performance_metrics_classification_file!=None) or (args.performance_metrics_regression_file!=None)):
        labels=predictions[1].values 
        tasks=predictions[1].columns 
        #if calibration has been used, we want accuracy metrics on the calibrated predictions (last entry in predictions list) 
        if ((args.calibrate_classification==True) or (args.calibrate_regression==True)): 
            predictions=predictions[-1]
        else:
            predictions=predictions[0]
            
        if args.performance_metrics_classification_file!=None:
            print("calculating classification performance metrics...")
            performance_metrics_classification=get_performance_metrics_classification(predictions,labels)
            print("writing classification performance metrics to file...") 
            write_performance_metrics(args.performance_metrics_classification_file,performance_metrics_classification,tasks) 
        elif args.performance_metrics_regression_file!=None:
            print("calculating regression performance metrics...") 
            performance_metrics_regression=get_performance_metrics_regression(predictions,labels)
            print("writing regression performance metrics to file...") 
            write_performance_metrics(args.performance_metrics_regression_file,performance_metrics_regression,tasks)
            
#write performance metrics to output file: 
def write_performance_metrics(output_file,metrics_dict,tasks):
    metrics_df=pd.DataFrame(metrics_dict,index=tasks).transpose()
    metrics_df.to_csv(output_file,sep='\t',header=True,index=True)
    
def main():
    args=parse_args()
    predict(args) 

    

if __name__=="__main__":
    main()
    



#Note: this is ugly w/ use of tf & K --> needed to avoid custom keras modifications 
import tensorflow as tf
import keras.backend as K

pseudocount=0.01


def contingency_table(y, z):
    """Note:  if y and z are not rounded to 0 or 1, they are ignored
    """
    y = K.cast(K.round(y), K.floatx())
    z = K.cast(K.round(z), K.floatx())
    
    def count_matches(y, z):
        return K.sum(K.cast(y, K.floatx()) * K.cast(z, K.floatx()))
    
    ones = K.ones_like(y)
    zeros = K.zeros_like(y)
    y_ones = K.equal(y, ones)
    y_zeros = K.equal(y, zeros)
    z_ones = K.equal(z, ones)
    z_zeros = K.equal(z, zeros)
    
    tp = count_matches(y_ones, z_ones)
    tn = count_matches(y_zeros, z_zeros)
    fp = count_matches(y_zeros, z_ones)
    fn = count_matches(y_ones, z_zeros)
    return (tp, tn, fp, fn)

def recall(y, z):
    """True positive rate `tp / (tp + fn)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return tp / (tp + fn+pseudocount)


def specificity(y, z):
    """True negative rate `tn / (tn + fp)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return tn / (tn + fp+pseudocount)


def fpr(y, z):
    """False positive rate `fp / (fp + tn)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return fp / (fp + tn+pseudocount)


def fnr(y, z):
    """False negative rate `fn / (fn + tp)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return fn / (fn + tp+pseudocount)


def precision(y, z):
    """Precision `tp / (tp + fp)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return tp / (tp + fp+pseudocount)


def fdr(y, z):
    """False discovery rate `fp / (tp + fp)`
    """
    tp, tn, fp, fn = contingency_table(y, z)
    return fp / (tp + fp+pseudocount)


def f1(y, z):
    """F1 score: `2 * (p * r) / (p + r)`, where p=precision and r=recall.
    """
    _recall = recall(y, z)
    _prec = precision(y, z)
    return 2 * (_prec * _recall) / (_prec + _recall+pseudocount)


def spearman_corr(y_true,y_pred):
    import K.contribs.metrics.streaming_pearson_correlation
    return K.contribs.metrics.streaming_pearson_correlation(y_pred,y_true)




from setuptools import setup,find_packages

config = {
    'include_package_data': True,
    'description': 'keras Accessibility Models (kerasAC)',
    'download_url': 'https://github.com/kundajelab/kerasAC',
    'version': '0.2',
    'packages': ['kerasAC'],
    'setup_requires': [],
    'install_requires': ['numpy>=1.9', 'keras>=2.2', 'h5py', 'pandas','pybigwig','deeplift','abstention'],
    'scripts': [],
    'entry_points': {'console_scripts': ['kerasAC_train = kerasAC.train:main',
                                         'kerasAC_predict=kerasAC.predict:main',
                                         'kerasAC_curves=kerasAC.curves:main',
                                         'kerasAC_interpret=kerasAC.interpret:main',
                                         'kerasAC_plot_interpretation=kerasAC.plot_interpretation:main',
                                         'kerasAC_cross_validate=kerasAC.cross_validate:main']},
    'name': 'kerasAC'
}

if __name__== '__main__':
    setup(**config)



#Draws PRC and ROC curves for predicted and true values

import argparse
from sklearn.metrics import precision_recall_curve,average_precision_score,roc_curve
import matplotlib.pyplot as plt
import h5py
import pickle
import numpy as np
from random import random
from operator import add
from pylab import rcParams
rcParams['figure.figsize'] = 10, 10

def parse_args():
    parser=argparse.ArgumentParser("generate PRC & ROC curves")
    parser.add_argument("--truth_hdf5")
    parser.add_argument("--prediction_pickle")
    parser.add_argument("--out_prefix")
    parser.add_argument("--labels")
    parser.add_argument("--title") 
    return parser.parse_args()

def filter_vals(precision,recall):
    thresholds=[i/100.0 for i in range(100,0,-1)]
    recall, precision = zip(*sorted(zip(recall, precision),reverse=True))
    recall=list(recall)
    precision=list(precision)
    rounded_recall=[round(i,2) for i in recall]
    filtered_precision=[]
    filtered_recall=[]
    cur_precision=0 
    for t in thresholds:
        if t in rounded_recall:            
            cur_index=rounded_recall.index(t)
            cur_precision=precision[cur_index]   
        filtered_precision.append(cur_precision)
        filtered_recall.append(t)
    return filtered_precision,filtered_recall

def main():
    args=parse_args()
    with open(args.prediction_pickle,'rb') as handle:
        y_pred=pickle.load(handle)
    y_true=h5py.File(args.truth_hdf5,'r')
    y_true=np.asarray(y_true['Y']['default_output_mode_name'])
    labels=open(args.labels,'r').read().strip().split('\n')[0].split('\t')[1::]
    mean_precision=[]
    mean_recall=[]
    mean_fpr=[]
    mean_tpr=[]

    #initialize figure
    fig1=plt.figure()
    ax1=fig1.add_subplot(121)
    ax2=fig1.add_subplot(122)
    #compute precision & recall for each task 
    for i in range(len(labels)):
        
        precision, recall, thresholds = precision_recall_curve(y_true[:,i], y_pred[:,i])
        fpr,tpr,thresholds=roc_curve(y_true[:,i],y_pred[:,i])
        
        cur_color=([random(),random(),random()])
        ax1.step(recall, precision, color=cur_color, alpha=0.2,where='post',label=labels[i])
        ax2.step(fpr,tpr,color=cur_color,alpha=0.2,where='post',label=labels[i])

        precision,recall=filter_vals(precision,recall)
        tpr,fpr=filter_vals(tpr,fpr)

        if len(mean_precision)==0:
            mean_precision=precision
        else:
            mean_precision=map(add, mean_precision,precision)
        if len(mean_recall)==0:
            mean_recall=recall
        else:
            mean_recall=map(add,mean_recall,recall)
        if len(mean_tpr)==0:
            mean_tpr=tpr
        else:
            mean_tpr=map(add,mean_tpr,tpr)
        if len(mean_fpr)==0:
            mean_fpr=fpr
        else:
            mean_fpr=map(add,mean_fpr,fpr)

    #get average metric values 
    mean_precision=[i/len(labels) for i in mean_precision]
    mean_recall=[i/len(labels) for i in mean_recall]
    mean_tpr=[i/len(labels) for i in mean_tpr]
    mean_fpr=[i/len(labels) for i in mean_fpr]
    
    ax1.step(mean_recall, mean_precision, color=(0,0,0), linewidth=2,
             where='post',label="mean")
    ax1.set_xlabel('Recall')
    ax1.set_ylabel('Precision')
    ax1.set_ylim([0.0, 1.05])
    ax1.set_xlim([0.0, 1.0])
    ax1.set_title(args.title)

    ax2.step(mean_fpr,mean_tpr,color=(0,0,0),linewidth=2,
             where='post',label='mean')
    ax2.set_xlabel('FPR')
    ax2.set_ylabel('TPR')
    ax2.set_ylim([0.0,1.05])
    ax2.set_xlim([0.0,1.0])
    ax2.set_title(args.title)

    #generate output file
    outf=open(args.out_prefix+'.average.metrics','w')
    outf.write('Precision\tRecall\tFPR\tTPR\n')
    for i in range(len(mean_precision)):
        outf.write(str(mean_precision[i])+'\t'+str(mean_recall[i])+'\t'+str(mean_fpr[i])+'\t'+str(mean_tpr[i])+'\n')
    #plt.savefig(args.out_prefix+".png",dpi=100)
    plt.show()
if __name__=="__main__":
    main()
    
